@@ -1,10 +1,10 @@
 
 #' Tutorial HTML dependency
-#' 
-#' @details HTML dependency for core tutorial JS and CSS. This should be included as a 
+#'
+#' @details HTML dependency for core tutorial JS and CSS. This should be included as a
 #' dependency for custom tutorial formats that wish to ensure that that
 #' tutorial.js and tutorial.css are loaded prior their own scripts and stylesheets.
-#' 
+#'
 #' @export
 tutorial_html_dependency <- function() {
   htmltools::htmlDependency(
@@ -46,13 +46,13 @@ html_dependency_src <- function(...) {
   }
 }
 
-
-localforage_html_dependency <- function() {
+idb_html_dependency <- function() {
   htmltools::htmlDependency(
-    name = "localforage",
-    version = "1.5",
-    src = system.file("lib/localforage", package = "learnr"),
-    script = "localforage.min.js"
+    name = "idb-keyvalue",
+    version = "3.2.0",
+    src = system.file("lib/idb-keyval", package = "learnr"),
+    script = "idb-keyval-iife-compat.min.js",
+    all_files = FALSE
   )
 }
 
@@ -83,4 +83,3 @@ ace_html_dependency <- function() {
     script = "ace.js"
   )
 }
-
